@@ -29,6 +29,7 @@ using System;
 using System.Collections;
 using System.IO;
 using System.Text;
+using System.Threading;
 using System.Xml;
 using System.Xml.Serialization;
 using Beagle.Util;
@@ -76,6 +77,9 @@ namespace Beagle {
 		// If applicable, otherwise set to null.
 		private String mimeType = null;
 
+		// The source backend that generated this indexable
+		private string source = null;
+		
 		// List of Property objects
 		private ArrayList properties = new ArrayList ();
 
@@ -229,6 +233,12 @@ namespace Beagle {
 			set { mimeType = value; }
 		}
 
+		[XmlAttribute]
+		public string Source {
+			get { return source; }
+			set { source = value; }
+		}
+
 		[XmlIgnore]
 		public bool IsNonTransient {
 			get { return ! DeleteContent && ContentUri.IsFile && ParentUri == null; }
@@ -442,6 +452,48 @@ namespace Beagle {
 			return writer.ToString ();
 		}
 
+		//////////////////////////
+
+		const int BUFFER_SIZE = 8192;
+
+		private static char [] GetCharBuffer ()
+		{
+			LocalDataStoreSlot slot;
+			slot = Thread.GetNamedDataSlot ("Char Buffer");
+
+			object obj;
+			char [] buffer;
+			obj = Thread.GetData (slot);
+			if (obj == null) {
+				buffer = new char [BUFFER_SIZE];
+				Thread.SetData (slot, buffer);
+			} else {
+				buffer = (char []) obj; 
+			}
+
+			return buffer;
+		}
+
+		private static byte [] GetByteBuffer ()
+		{
+			LocalDataStoreSlot slot;
+			slot = Thread.GetNamedDataSlot ("Byte Buffer");
+
+			object obj;
+			byte [] buffer;
+			obj = Thread.GetData (slot);
+			if (obj == null) {
+				buffer = new byte [BUFFER_SIZE];
+				Thread.SetData (slot, buffer);
+			} else {
+				buffer = (byte []) obj; 
+			}
+
+			return buffer;
+		}
+
+		//////////////////////////
+
 		private static Uri TextReaderToTempFileUri (TextReader reader)
 		{
 			if (reader == null)
@@ -457,17 +509,18 @@ namespace Beagle {
 			// Make sure the temporary file is only readable by the owner.
 			// FIXME: There is probably a race here.  Could some malicious program
 			// do something to the file between creation and the chmod?
-			Mono.Posix.Syscall.chmod (filename, (Mono.Posix.FileMode) 256);
+			Mono.Unix.Native.Syscall.chmod (filename, (Mono.Unix.Native.FilePermissions) 256);
 
 			BufferedStream bufferedStream = new BufferedStream (fileStream);
 			StreamWriter writer = new StreamWriter (bufferedStream);
 
-			const int BUFFER_SIZE = 8192;
-			char [] buffer = new char [BUFFER_SIZE];
+
+			char [] buffer;
+			buffer = GetCharBuffer ();
 
 			int read;
 			do {
-				read = reader.Read (buffer, 0, BUFFER_SIZE);
+				read = reader.Read (buffer, 0, buffer.Length);
 				if (read > 0)
 					writer.Write (buffer, 0, read);
 			} while (read > 0);
@@ -492,16 +545,16 @@ namespace Beagle {
 			// Make sure the temporary file is only readable by the owner.
 			// FIXME: There is probably a race here.  Could some malicious program
 			// do something to the file between creation and the chmod?
-			Mono.Posix.Syscall.chmod (filename, (Mono.Posix.FileMode) 256);
+			Mono.Unix.Native.Syscall.chmod (filename, (Mono.Unix.Native.FilePermissions) 256);
 
 			BufferedStream bufferedStream = new BufferedStream (fileStream);
 
-			const int BUFFER_SIZE = 8192;
-			byte [] buffer = new byte [BUFFER_SIZE];
+			byte [] buffer;
+			buffer = GetByteBuffer ();
 
 			int read;
 			do {
-				read = stream.Read (buffer, 0, BUFFER_SIZE);
+				read = stream.Read (buffer, 0, buffer.Length);
 				if (read > 0)
 					bufferedStream.Write (buffer, 0, read);
 			} while (read > 0);
