@@ -52,6 +52,9 @@ namespace Best {
 
 	public class BestWindow : Gtk.Window {
 
+		Gtk.AccelGroup accel_group;
+		GlobalKeybinder global_keys;
+
 		public BestWindow (string query) : base (WindowType.Toplevel)
 		{
 			CreateWindow (query);
@@ -62,19 +65,17 @@ namespace Best {
 			CreateWindow (null);
 		}
 		
-		public void FocusEntry () {
-			//entry.SelectRegion (0, -1);
+		public void FocusEntry ()
+		{
 			entry.GrabFocus ();
 		}
 
-		public void FocusEntryHandler (object o, EventArgs args) {
+		public void FocusEntryHandler (object o, EventArgs args)
+		{
 			FocusEntry ();
 		}
 
-		Gtk.AccelGroup accel_group;
-		GlobalKeybinder global_keys;
-
-		void CreateWindow (string query)
+		private void CreateWindow (string query)
 		{
 			Title = Best.DefaultWindowTitle;
 
@@ -140,10 +141,7 @@ namespace Best {
 						    0,
 						    Gtk.AccelFlags.Visible);
 
-			//DBusisms.BeagleDown += OnBeagleDown;
-
 			UpdateFromConf ();
-			
 			UpdatePage ();
 
 			if (query != null)
@@ -193,12 +191,6 @@ namespace Best {
 
 		}
 
-		private void OnBeagleDown ()
-		{
-			SetBusy (false);
-			DetachQuery ();
-		}
-
 		private void DoDelete (object o, DeleteEventArgs args)
 		{
 			Hide ();
@@ -444,11 +436,11 @@ namespace Best {
 			}
 		}
 
-		public void StoreSettingsInConf ()
+		public void StoreSettingsInConf (bool with_tray)
 		{
 			Console.WriteLine ("Storing setting in Config");
 			int pos_x = posX, pos_y = posY;
-			if (WindowIsVisible)
+			if (with_tray && WindowIsVisible)
 				GetPosition (out pos_x, out pos_y);
 			int width = 0, height = 0;
 			GetSize (out width, out height);
@@ -501,7 +493,6 @@ namespace Best {
 		private bool RunDelayedQuery ()
 		{
 			if (delayedQuery != null) {
-				string tmp = delayedQuery;
 				delayedQuery = null;
 				System.Console.WriteLine ("Delayed query fired");
 				StartQuery ();
@@ -562,17 +553,11 @@ namespace Best {
 			SetBusy (false);
 		}
 
-		private void OnCancelled (CancelledResponse response)
-		{
-			SetBusy (false);
-		}
-
 		private void AttachQuery ()
 		{
 			query.HitsAddedEvent += OnHitsAdded;
 			query.HitsSubtractedEvent += OnHitsSubtracted;
 			query.FinishedEvent += OnFinished;
-			query.CancelledEvent += OnCancelled;
 		}
 
 		private void DetachQuery ()
@@ -581,7 +566,6 @@ namespace Best {
 				query.HitsAddedEvent -= OnHitsAdded;
 				query.HitsSubtractedEvent -= OnHitsSubtracted;
 				query.FinishedEvent -= OnFinished;
-				query.CancelledEvent -= OnCancelled;
 				query.Close ();
 
 				query = null;
@@ -640,6 +624,8 @@ namespace Best {
 			root.Clear ();
 			UpdatePage ();
 			entry.Text = "";
+			// clear search is also "stop current search" - implicitly
+			DetachQuery ();
 		}
 
 		private void StoreSearch (string query)
