@@ -48,7 +48,7 @@ namespace Beagle.Daemon
 
 		static Hashtable remap_table = new Hashtable ();
 
-		static string arg_output, arg_tag;
+		static string arg_output, arg_tag, arg_source;
 
 		/////////////////////////////////////////////////////////
 		
@@ -70,6 +70,17 @@ namespace Beagle.Daemon
 		/////////////////////////////////////////////////////////
 		
 		static void Main (string [] args)
+		{
+			try {
+				DoMain (args);
+			} catch (Exception ex) {
+				Logger.Log.Error ("Unhandled exception thrown.  Exiting immediately.");
+				Logger.Log.Error (ex);
+				Environment.Exit (1);
+			}
+		}
+
+		static void DoMain (string [] args)
 		{
 			if (args.Length < 2)
 				PrintUsage ();
@@ -162,6 +173,14 @@ namespace Beagle.Daemon
 					arg_disable_restart = true;
 					break;
 
+				case "--source":
+					if (next_arg == null)
+						break;
+
+					arg_source = next_arg;
+					++i;
+					break;
+
 				default:
 					string path = Path.IsPathRooted (arg) ? arg : Path.GetFullPath (arg);
 					
@@ -284,6 +303,26 @@ namespace Beagle.Daemon
 		
 		/////////////////////////////////////////////////////////////////
 
+		static void AddToRequest (IndexerRequest request, Indexable indexable)
+		{
+			// Disable filtering and only index file attributes
+			if (arg_disable_filtering)
+				indexable.Filtering = IndexableFiltering.Never;
+					
+			// Tag the item for easy identification (for say, removal)
+			if (arg_tag != null)
+				indexable.AddProperty (Property.NewKeyword("Tag", arg_tag));
+
+			if (arg_source == null) {
+				DirectoryInfo dir = new DirectoryInfo (StringFu.SanitizePath (arg_output));
+				arg_source = dir.Name;
+			}
+
+			indexable.Source = arg_source;
+
+			request.Add (indexable);
+		}
+
 		static IndexerReceipt [] FlushIndexer (IIndexer indexer, IndexerRequest request)
 		{
 			IndexerReceipt [] receipts;
@@ -325,7 +364,7 @@ namespace Beagle.Daemon
 
 			request.Clear (); // clear out the old request
 			foreach (Indexable i in pending_children) // and then add the children
-				request.Add (i);
+				AddToRequest (request, i);
 			
 			return receipts;
 		}
@@ -350,18 +389,10 @@ namespace Beagle.Daemon
 					// Create the indexable and add the standard properties we
 					// use in the FileSystemQueryable.
 					indexable = new Indexable (uri);
-					indexable.Timestamp = file.LastWriteTime;
+					indexable.Timestamp = file.LastWriteTimeUtc;
 					FSQ.AddStandardPropertiesToIndexable (indexable, file.Name, Guid.Empty, false);
-
-					// Disable filtering and only index file attributes
-					if (arg_disable_filtering)
-						indexable.Filtering = IndexableFiltering.Never;
 					
-					// Tag the item for easy identification (for say, removal)
-					if (arg_tag != null)
-						indexable.AddProperty (Property.NewKeyword("Tag", arg_tag));
-					
-					pending_request.Add (indexable);
+					AddToRequest (pending_request, indexable);
 					
 					if (pending_request.Count >= BATCH_SIZE) {
 						Logger.Log.Debug ("Flushing driver, {0} items in queue", pending_request.Count);
@@ -424,25 +455,16 @@ namespace Beagle.Daemon
 		
 		// From BeagleDaemon.cs
 
-                // The integer values of the Mono.Posix.Signal enumeration don't actually
-		// match the Linux signal numbers of Linux.  Oops!
-		// This is fixed in Mono.Unix, but for the moment we want to maintain
-		// compatibility with mono 1.0.x.
-		const int ACTUAL_LINUX_SIGINT  = 2;
-		const int ACTUAL_LINUX_SIGQUIT = 3;
-		const int ACTUAL_LINUX_SIGTERM = 15;
-		
 		static void SetupSignalHandlers ()
 		{
 			// Force OurSignalHandler to be JITed
 			OurSignalHandler (-1);
 			
 			// Set up our signal handler
-			Mono.Posix.Syscall.sighandler_t sig_handler;
-			sig_handler = new Mono.Posix.Syscall.sighandler_t (OurSignalHandler);
-                        Mono.Posix.Syscall.signal (ACTUAL_LINUX_SIGINT, sig_handler);
-                        Mono.Posix.Syscall.signal (ACTUAL_LINUX_SIGQUIT, sig_handler);
-                        Mono.Posix.Syscall.signal (ACTUAL_LINUX_SIGTERM, sig_handler);
+			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGINT, OurSignalHandler);
+			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGTERM, OurSignalHandler);
+			if (Environment.GetEnvironmentVariable("BEAGLE_THERE_BE_NO_QUITTIN") == null)
+				Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGQUIT, OurSignalHandler);
 		}
 		
 		static void OurSignalHandler (int signal)
@@ -468,6 +490,7 @@ namespace Beagle.Daemon
 			usage += 
 				"Usage: beagle-build-index [OPTIONS] --target <index_path> <path> [path]\n\n" +
 				"Options:\n" +
+				"  --source [name]\t\tThe index's source name.  Defaults to the target directory name\n" +
 				"  --remap [path1:path2]\t\tRemap data paths to fit target. \n" +
 				"  --tag [tag]\t\t\tTag index data for identification.\n" + 
 				"  --recursive\t\t\tCrawl source path recursivly.\n" + 
