@@ -36,7 +36,6 @@ using Beagle.Util;
 using Beagle.Daemon;
 
 using Camel = Beagle.Util.Camel;
-using Mono.Posix;
 
 namespace Beagle.Daemon.EvolutionMailDriver {
 
@@ -44,7 +43,15 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 		private static bool gmime_initialized = false;
 
-		private EvolutionMailQueryable queryable;
+		private static ArrayList excludes = new ArrayList ();
+
+		static EvolutionMailIndexableGenerator () {
+			foreach (ExcludeItem exclude in Conf.Indexing.Excludes)
+				if (exclude.Type == ExcludeType.MailFolder)
+					excludes.Add (exclude);
+		}
+
+		protected EvolutionMailQueryable queryable;
 
 		protected string account_name, folder_name;
 		protected int count, indexed_count;
@@ -62,8 +69,9 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		public abstract Indexable GetNextIndexable ();
 		public abstract void Checkpoint ();
 
-		protected EvolutionMailQueryable Queryable {
-			get { return this.queryable; }
+		public void PostFlushHook ()
+		{
+			Checkpoint ();
 		}
 
 		protected static void InitializeGMime ()
@@ -81,149 +89,23 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			else
 				return false;
 		}
-					
-		protected bool CrawlNeeded ()
+
+		protected bool IgnoreFolder (string path)
 		{
-			string timeStr;
-
-			try {
-				timeStr = ExtendedAttribute.Get (this.CrawlFile.FullName, "LastCrawl");
-			} catch {
-				EvolutionMailQueryable.log.Debug ("Unable to get last crawl time on {0}",
-								  this.CrawlFile.FullName);
-				return true;
+			// FIXME: Use System.IO.Path
+			foreach (ExcludeItem exclude in excludes) {
+				if (exclude.IsMatch (path))
+					return true;
 			}
-
-			DateTime lastCrawl = StringFu.StringToDateTime (timeStr);
-
-			if (this.CrawlFile.LastWriteTime > lastCrawl)
-				return true;
-			else
-				return false;
+			return false;
 		}
-
+					
 		protected void CrawlFinished ()
 		{
-			try {
-				ExtendedAttribute.Set (this.CrawlFile.FullName, "LastCrawl",
-						       StringFu.DateTimeToString (DateTime.Now));
-			} catch {
-				EvolutionMailQueryable.log.Debug ("Unable to set last crawl time on {0}",
-								  this.CrawlFile.FullName);
-			}
-		}
+			// FIXME: This is a little sketchy
+			this.queryable.FileAttributesStore.AttachLastWriteTime (this.CrawlFile.FullName, DateTime.UtcNow);
 
-		static public bool SupportedContentType (GMime.ContentType type)
-		{
-			return type.Type == "text" || type.Type == "Text" || type.Type == "TEXT";
-		}
-
-		protected class PartHandler {
-			private MultiReader reader = null;
-			private int depth = 0;
-
-			public void OnEachPart (GMime.Object part)
-			{
-				if (this.reader == null)
-					this.reader = new MultiReader ();
-
-				//for (int i = 0; i < depth; i++)
-				//  Console.Write ("  ");
-				//Console.WriteLine ("Content-Type: {0}", part.ContentType);
-
-				++depth;
-
-				if (part is GMime.MessagePart) {
-					GMime.MessagePart msg_part = (GMime.MessagePart) part;
-
-					using (GMime.Message message = msg_part.Message) {
-						using (GMime.Object subpart = message.MimePart)
-							this.OnEachPart (subpart);
-					}
-				} else if (part is GMime.Multipart) {
-					GMime.Multipart multipart = (GMime.Multipart) part;
-
-					int num_parts = multipart.Number;
-					for (int i = 0; i < num_parts; i++) {
-						using (GMime.Object subpart = multipart.GetPart (i))
-							this.OnEachPart (subpart);
-					}
-				} else if (SupportedContentType (part.ContentType)) {
-					MemoryStream stream = new MemoryStream (part.GetData ());
-					StreamReader reader = new StreamReader (stream);
-
-					// If this is the first part, we need to skip past the
-					// message headers --- we want to store that data in
-					// properties, not index it as text.
-					if (this.reader.Count == 0) {
-						string line;
-						do {
-							line = reader.ReadLine ();
-						} while (line != null && line != "");
-					}
-
-					this.reader.Add (reader);
-				} 
-
-				--depth;
-			}
-
-			public static MultiReader GetReader (GMime.Message message)
-			{
-				// FIXME: This will work for now, but really we need
-				// to make it so that we can index multiple things
-				// from a single indexable.
-				PartHandler handler = new PartHandler ();
-				//message.ForeachPart (new GMime.PartFunc (handler.OnEachPart));
-				using (GMime.Object mime_part = message.MimePart)
-					handler.OnEachPart (mime_part);
-				return handler.reader;
-			}
-
-		}
-
-		protected static Stream ReadAppData (string name)
-		{
-			string path = Path.Combine (Path.Combine (PathFinder.StorageDir, "MailIndex"), name);
-			return new FileStream (path, System.IO.FileMode.Open, FileAccess.Read, FileShare.Read);
-		}
-
-		protected static string ReadAppDataLine (string name)
-		{
-			Stream stream;
-
-			try {
-				stream = ReadAppData (name);
-			} catch (FileNotFoundException) {
-				return null;
-			}
-
-			StreamReader sr = new StreamReader (stream);
-			string line = sr.ReadLine ();
-			sr.Close ();
-			return line;
-		}
-
-		protected static Stream WriteAppData (string name)
-		{
-			string path = Path.Combine (Path.Combine (PathFinder.StorageDir, "MailIndex"), name);
-			return new FileStream (path, System.IO.FileMode.Create, FileAccess.Write, FileShare.None);
-		}
-
-		protected static void WriteAppDataLine (string name, string line)
-		{
-			if (line == null) {
-				string path = Path.Combine (Path.Combine (PathFinder.StorageDir, "MailIndex"), name);
-
-				if (File.Exists (path))
-					File.Delete (path);
-
-				return;
-			}
-
-			StreamWriter sw = new StreamWriter (WriteAppData (name));
-			sw.WriteLine (line);
-			sw.Close ();
+			this.queryable.SetGeneratorProgress (this, 100);
 		}
 
 		public string StatusName {
@@ -250,49 +132,6 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		{
 			return this.CrawlFile.FullName.GetHashCode ();
 		}
-
-		// FIXME FIXME FIXME!
-		// This is a fragile piece-of-shit parser.
-		// This will not do the right thing if the address is quoted
-		// or contains special characters.  For example, we will do the wrong thing
-		// with:
-		// "Foo, bar" <foo@bar.com>, Zoo <zoo@bar.com>"
-		// Also, this almost certainly won't deal well w/ broken addresses.
-		static protected ICollection ExtractAddresses (string addr_str)
-		{
-			string [] split = addr_str.Split (',');
-
-			ArrayList addresses = new ArrayList ();
-			foreach (string part in split) {
-				if (part == "")
-					continue;
-				if (part [part.Length - 1] == '>') {
-					int i = part.LastIndexOf ('<');
-					addresses.Add (part.Substring (i+1, part.Length-i-2));
-				} else if (part [part.Length - 1] == ')') {
-					int i = part.LastIndexOf ('(');
-					addresses.Add (part.Substring (i+1, part.Length-i-2));
-				} else {
-					addresses.Add (part);
-				}
-			}
-			
-			return addresses;
-		}
-
-		static protected void MapAddressStringToProperties (Indexable indexable,
-								    string    property_key,
-								    string    address_string)
-		{
-			if (address_string == null || address_string.Length == 0)
-				return;
-
-			foreach (string one_addr in ExtractAddresses (address_string)) {
-				Property prop;
-				prop = Property.NewKeyword (property_key, one_addr);
-				indexable.AddProperty (prop);
-			}
-		}
 	}
 
 	public class EvolutionMailIndexableGeneratorMbox : EvolutionMailIndexableGenerator {
@@ -300,6 +139,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		private int mbox_fd = -1;
 		private GMime.StreamFs mbox_stream;
 		private GMime.Parser mbox_parser;
+		private long file_size;
 
 		public EvolutionMailIndexableGeneratorMbox (EvolutionMailQueryable queryable, FileInfo mbox_info) : base (queryable)
 		{
@@ -333,33 +173,37 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 			if (this.IsSpamFolder (this.folder_name))
 				return false;
-			
+
+			if (this.IgnoreFolder (this.mbox_info.FullName))
+				return false;
+
 			return true;
 		}
 
 		private long MboxLastOffset {
 			get {
-				string offset_str = ReadAppDataLine ("offset-" + this.folder_name.Replace ('/', '-'));
+				string offset_str = this.queryable.ReadDataLine ("offset-" + this.folder_name.Replace ('/', '-'));
 				long offset = Convert.ToInt64 (offset_str);
 
-				Logger.Log.Debug ("mbox {0} offset is {1}", this.mbox_info.Name, offset);
 				return offset;
 			}
 
 			set {
-				WriteAppDataLine ("offset-" + this.folder_name.Replace ('/', '-'), value.ToString ());
+				this.queryable.WriteDataLine ("offset-" + this.folder_name.Replace ('/', '-'), value.ToString ());
 			}
 		}
 
 		public override bool HasNextIndexable ()
 		{
 			if (this.account_name == null) {
-				if (!Setup ())
+				if (!Setup ()) {
+					this.queryable.RemoveGeneratorProgress (this);
 					return false;
+				}
 			}
 
 			if (this.mbox_fd < 0) {
-				Logger.Log.Debug ("opening mbox {0}", this.mbox_info.Name);
+				Logger.Log.Debug ("Opening mbox {0}", this.mbox_info.Name);
 
 				try {
 					InitializeGMime ();
@@ -369,11 +213,14 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 					return false;
 				}
 
-				this.mbox_fd = Syscall.open (this.mbox_info.FullName, OpenFlags.O_RDONLY);
+				this.mbox_fd = Mono.Unix.Native.Syscall.open (this.mbox_info.FullName, Mono.Unix.Native.OpenFlags.O_RDONLY);
 				this.mbox_stream = new GMime.StreamFs (this.mbox_fd);
 				this.mbox_stream.Seek ((int) this.MboxLastOffset);
 				this.mbox_parser = new GMime.Parser (this.mbox_stream);
 				this.mbox_parser.ScanFrom = true;
+
+				FileInfo info = new FileInfo (this.mbox_info.FullName);
+				this.file_size = info.Length;
 			}
 
 			if (this.mbox_parser.Eos ()) {
@@ -387,8 +234,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				this.mbox_parser.Dispose ();
 				this.mbox_parser = null;
 				
-				EvolutionMailQueryable.log.Debug ("{0}: Finished indexing {1} messages",
-								  this.folder_name, this.indexed_count);
+				Logger.Log.Debug ("{0}: Finished indexing {1} messages", this.folder_name, this.indexed_count);
 
 				this.MboxLastOffset = offset;
 				this.CrawlFinished ();
@@ -418,116 +264,130 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				
 				string x_evolution = message.GetHeader ("X-Evolution");
 				if (x_evolution == null || x_evolution == "") {
-					EvolutionMailQueryable.log.Info ("{0}: Message at offset {1} has no X-Evolution header!",
-									 this.folder_name, this.mbox_parser.FromOffset);
+					Logger.Log.Info ("{0}: Message at offset {1} has no X-Evolution header!",
+							 this.folder_name, this.mbox_parser.FromOffset);
 					return null;
 				}
 				
-				string uid_str = x_evolution.Substring (0, x_evolution.IndexOf ('-'));
+				int separator_idx = x_evolution.IndexOf ('-');
+
+				string uid_str = x_evolution.Substring (0, separator_idx);
 				string uid = Convert.ToUInt32 (uid_str, 16).ToString (); // ugh.
+				uint flags = Convert.ToUInt32 (x_evolution.Substring (separator_idx + 1), 16);
+
+				Indexable indexable = this.GMimeMessageToIndexable (uid, message, flags);
 				
-				Indexable indexable = this.GMimeMessageToIndexable (uid, message);
+				if (indexable == null)
+					return null;
+
 				++this.indexed_count;
 
-				// HACK: update your recipients
-				EvolutionMailQueryable.AddAsYourRecipient (indexable);
-				
 				return indexable;
 			}
 		}
 
-		private Indexable GMimeMessageToIndexable (string uid, GMime.Message message)
+		private static bool CheckFlags (uint flags, Camel.CamelFlags test)
 		{
+			return (flags & (uint) test) == (uint) test;
+		}
+
+		private Indexable GMimeMessageToIndexable (string uid, GMime.Message message, uint flags)
+		{
+			// Don't index messages flagged as junk
+			if (CheckFlags (flags, Camel.CamelFlags.Junk))
+				return null;
+
 			System.Uri uri = EvolutionMailQueryable.EmailUri (this.account_name, this.folder_name, uid);
 			Indexable indexable = new Indexable (uri);
 
-			indexable.Timestamp = message.Date;
-			indexable.Type = "MailMessage";
-			indexable.MimeType = "text/plain";
+			indexable.Timestamp = message.Date.ToUniversalTime ();
+			indexable.HitType = "MailMessage";
+			indexable.MimeType = "message/rfc822";
 			indexable.CacheContent = false;
 
 			indexable.AddProperty (Property.NewKeyword ("fixme:client", "evolution"));
-
-			string subject = GMime.Utils.HeaderDecodePhrase (message.Subject);
-			indexable.AddProperty (Property.New ("dc:title", subject));
-			indexable.AddProperty (Property.New ("fixme:subject", subject));
-
 			indexable.AddProperty (Property.NewKeyword ("fixme:account", "Local"));
                         indexable.AddProperty (Property.NewKeyword ("fixme:folder", this.folder_name));
 
-			string to_str = message.GetRecipientsAsString (GMime.Message.RecipientType.To);
-			string cc_str = message.GetRecipientsAsString (GMime.Message.RecipientType.Cc);
-			string from_str = GMime.Utils.HeaderDecodePhrase (message.Sender);
-                        indexable.AddProperty (Property.NewKeyword ("fixme:to", to_str));       
-			indexable.AddProperty (Property.NewKeyword ("fixme:cc", cc_str));
-			indexable.AddProperty (Property.NewKeyword ("fixme:from", from_str));
+			GMime.InternetAddressList addrs;
 
-			if (this.folder_name == "Sent") {
+			addrs = message.GetRecipients (GMime.Message.RecipientType.To);
+			foreach (GMime.InternetAddress ia in addrs) {
+				if (this.folder_name == "Sent")
+					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
+			}
+			addrs.Dispose ();
+
+			addrs = message.GetRecipients (GMime.Message.RecipientType.Cc);
+			foreach (GMime.InternetAddress ia in addrs) {
+				if (this.folder_name == "Sent")
+					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
+			}
+			addrs.Dispose ();
+
+			addrs = GMime.InternetAddressList.ParseString (GMime.Utils.HeaderDecodePhrase (message.Sender));
+			foreach (GMime.InternetAddress ia in addrs) {
+				if (this.folder_name != "Sent")
+					indexable.AddProperty (Property.NewKeyword ("fixme:gotFrom", ia.Addr));
+			}
+			addrs.Dispose ();
+
+			if (this.folder_name == "Sent")
 				indexable.AddProperty (Property.NewFlag ("fixme:isSent"));
 
-				MapAddressStringToProperties (indexable, "fixme:sentTo", to_str);
-				MapAddressStringToProperties (indexable, "fixme:sentTo", cc_str);
-			} else {
-				MapAddressStringToProperties (indexable, "fixme:gotFrom", from_str);
-			}
-
-			if (message.MimePart is GMime.Multipart || message.MimePart is GMime.MessagePart)
-				indexable.AddProperty (Property.NewFlag ("fixme:hasAttachments"));
-
-			string list_id = message.GetHeader ("List-Id");
-
-			if (list_id != null) {
-				// FIXME: Might need some additional parsing.
-				indexable.AddProperty (Property.NewKeyword ("fixme:mlist", GMime.Utils.HeaderDecodePhrase (list_id)));
-			}
-
+			//indexable.AddProperty (Property.NewDate ("fixme:date", message.Date));
+#if false
 			if (this.folder_name == "Sent")
 				indexable.AddProperty (Property.NewDate ("fixme:sentdate", message.Date));
 			else
 				indexable.AddProperty (Property.NewDate ("fixme:received", message.Date));
-			
-#if false
-			// FIXME - These are stored in the Evolution summary
-			// and we don't parse that anymore.
-                        indexable.AddProperty (Property.NewKeyword ("fixme:flags",    messageInfo.flags));
-
-			if (messageInfo.received != DateTime.MinValue)
-				indexable.AddProperty (Property.NewDate ("fixme:received", messageInfo.received));
-
-			if (messageInfo.sent != DateTime.MinValue)
-				indexable.AddProperty (Property.NewDate ("fixme:sentdate", messageInfo.sent));
-
-			if (messageInfo.IsAnswered)
-				indexable.AddProperty (Property.NewFlag ("fixme:isAnswered"));
-
-			if (messageInfo.IsDeleted)
-				indexable.AddProperty (Property.NewFlag ("fixme:isDeleted"));
-
-			if (messageInfo.IsDraft)
-				indexable.AddProperty (Property.NewFlag ("fixme:isDraft"));
-
-			if (messageInfo.IsFlagged)
-				indexable.AddProperty (Property.NewFlag ("fixme:isFlagged"));
-
-			if (messageInfo.IsSeen)
-				indexable.AddProperty (Property.NewFlag ("fixme:isSeen"));
-
-			if (messageInfo.IsAnsweredAll)
-				indexable.AddProperty (Property.NewFlag ("fixme:isAnsweredAll"));
 #endif
 
-			indexable.SetTextReader (PartHandler.GetReader (message));
+			indexable.AddProperty (Property.NewKeyword ("fixme:flags", flags));
+
+			if (CheckFlags (flags, Camel.CamelFlags.Answered))
+				indexable.AddProperty (Property.NewFlag ("fixme:isAnswered"));
+
+			if (CheckFlags (flags, Camel.CamelFlags.Deleted))
+				indexable.AddProperty (Property.NewFlag ("fixme:isDeleted"));
+
+			if (CheckFlags (flags, Camel.CamelFlags.Draft))
+				indexable.AddProperty (Property.NewFlag ("fixme:isDraft"));
+
+			if (CheckFlags (flags, Camel.CamelFlags.Flagged))
+				indexable.AddProperty (Property.NewFlag ("fixme:isFlagged"));
+
+			if (CheckFlags (flags, Camel.CamelFlags.Seen))
+				indexable.AddProperty (Property.NewFlag ("fixme:isSeen"));
+
+			if (CheckFlags (flags, Camel.CamelFlags.AnsweredAll))
+				indexable.AddProperty (Property.NewFlag ("fixme:isAnsweredAll"));
+
+			MemoryStream stream = new MemoryStream ();
+			message.WriteToStream (stream);
+			stream.Seek (0, SeekOrigin.Begin);
+			indexable.SetBinaryStream (stream);
 
 			return indexable;
 		}
 
 		public override void Checkpoint ()
 		{
-			EvolutionMailQueryable.log.Debug ("{0}: indexed {1} messages",
-							  this.folder_name, this.indexed_count);
+			long offset = -1;
 
 			if (this.mbox_parser != null)
-				this.MboxLastOffset = this.mbox_parser.FromOffset;
+				this.MboxLastOffset = offset = this.mbox_parser.FromOffset;
+
+			string progress = "";
+			if (this.file_size > 0 && offset > 0) {
+				progress = String.Format (" ({0}/{1} bytes {2:###.0}%)",
+							  offset, this.file_size, 100.0 * offset / this.file_size);
+
+				this.queryable.SetGeneratorProgress (this, (int) (100.0 * offset / this.file_size));
+			}
+
+			Logger.Log.Debug ("{0}: indexed {1} messages{2}",
+					  this.folder_name, this.indexed_count, progress);
 		}
 
 		public override string GetTarget ()
@@ -601,8 +461,8 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			try {
 				this.accounts = (ICollection) GConfThreadHelper.Get ("/apps/evolution/mail/accounts");
 			} catch (Exception ex) {
-				EvolutionMailQueryable.log.Warn ("Caught exception in Setup(): " + ex.Message);
-				EvolutionMailQueryable.log.Warn ("There are no configured evolution accounts, ignoring {0}", this.imap_name);
+				Logger.Log.Warn ("Caught exception in Setup(): " + ex.Message);
+				Logger.Log.Warn ("There are no configured evolution accounts, ignoring {0}", this.imap_name);
 				return false;
 			}
 
@@ -647,6 +507,8 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				if (semicolon != -1)
 					imap_url = imap_url.Substring (0, semicolon) + imap_url.Substring (user_end);
 
+				// Escape backslashes, which frequently appear when using IMAP against Exchange servers
+				this.imap_name = this.imap_name.Replace ("\\", "%5c");
 
 				// Escape out additional @s in the name.  I hate the class libs so much.
 				int lastIdx = this.imap_name.LastIndexOf ('@');
@@ -667,8 +529,8 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				}
 			}
 
-			if (account_name == null) {
-				EvolutionMailQueryable.log.Info ("Unable to determine account name for {0}", this.imap_name);
+			if (this.account_name == null) {
+				Logger.Log.Info ("Unable to determine account name for {0}", this.imap_name);
 				return false;
 			}
 
@@ -677,6 +539,10 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			DirectoryInfo dir_info = new DirectoryInfo (dir_name);
 			if (this.IsSpamFolder (dir_info.Name))
 				return false;
+
+			// Check if the folder is listed in the configuration as to be excluded from indexing
+			if (this.IgnoreFolder (dir_info.FullName))
+				return false;
 					
 			this.folder_name = GetFolderName (new DirectoryInfo (dir_name));
 
@@ -685,6 +551,9 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 		private string FolderCacheName {
 			get {
+				if (this.account_name == null || this.folder_name == null)
+					return null;
+
 				if (this.folder_cache_name == null)
 					this.folder_cache_name = "status-" + this.account_name + "-" + this.folder_name.Replace ('/', '-');
 
@@ -697,12 +566,17 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			Stream cacheStream;
 			BinaryFormatter formatter;
 
+			if (this.FolderCacheName == null) {
+				this.mapping = new Hashtable ();
+				return false;
+			}
+
 			try {
-				cacheStream = ReadAppData (this.FolderCacheName);
+				cacheStream = this.queryable.ReadDataStream (this.FolderCacheName);
 				formatter = new BinaryFormatter ();
 				this.mapping = formatter.Deserialize (cacheStream) as Hashtable;
 				cacheStream.Close ();
-				EvolutionMailQueryable.log.Debug ("Successfully loaded previous crawled data from disk: {0}", this.FolderCacheName);
+				Logger.Log.Debug ("Successfully loaded previous crawled data from disk: {0}", this.FolderCacheName);
 
 				return true;
 			} catch {
@@ -716,8 +590,11 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		{
 			Stream cacheStream;
 			BinaryFormatter formatter;
+
+			if (this.FolderCacheName == null)
+				return;
 			
-			cacheStream = WriteAppData (this.FolderCacheName);
+			cacheStream = this.queryable.WriteDataStream (this.FolderCacheName);
 			formatter = new BinaryFormatter ();
 			formatter.Serialize (cacheStream, mapping);
 			cacheStream.Close ();
@@ -726,18 +603,23 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		public override bool HasNextIndexable ()
 		{
 			if (this.account_name == null) {
-				if (!Setup ())
+				if (!Setup ()) {
+					this.queryable.RemoveGeneratorProgress (this);
 					return false;
+				}
 			}
 
 			if (this.mapping == null) {
 				bool cache_loaded = this.LoadCache ();
 
 				this.deleted_list = new ArrayList (this.mapping.Keys);
+				this.deleted_list.Sort ();
+				Logger.Log.Debug ("Deleted list starting at {0} for {1}", this.deleted_list.Count, this.folder_name);
 
 				// Check to see if we even need to bother walking the summary
-				if (cache_loaded && ! CrawlNeeded ()) {
-					EvolutionMailQueryable.log.Debug ("{0}: summary has not been updated; crawl unncessary", this.folder_name);
+				if (cache_loaded && this.queryable.FileAttributesStore.IsUpToDate (this.CrawlFile.FullName)) {
+					Logger.Log.Debug ("{0}: summary has not been updated; crawl unncessary", this.folder_name);
+					this.queryable.RemoveGeneratorProgress (this);
 					return false;
 				}
 			}
@@ -749,8 +631,9 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 					else
 						this.summary = Camel.Summary.LoadImap4Summary (this.summary_info.FullName);
 				} catch (Exception e) {
-					EvolutionMailQueryable.log.Warn ("Unable to index {0}: {1}", this.folder_name,
+					Logger.Log.Warn ("Unable to index {0}: {1}", this.folder_name,
 									 e.Message);
+					this.queryable.RemoveGeneratorProgress (this);
 					return false;
 				}
 			}
@@ -761,20 +644,26 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			if (this.summary_enumerator.MoveNext ())
 				return true;
 
+			Logger.Log.Debug ("Queuing up {0} removals for deleted messages in ", this.deleted_list.Count, this.folder_name);
 			foreach (string uid in this.deleted_list) {
 				Uri uri = EvolutionMailQueryable.EmailUri (this.account_name, this.folder_name, uid);
 
 				// FIXME: This is kind of a hack, but it's the only way
 				// with the IndexableGenerator to handle our removals.
-				Scheduler.Task task = this.Queryable.NewRemoveTask (uri);
+				Scheduler.Task task = this.queryable.NewRemoveTask (uri);
 				task.Priority = Scheduler.Priority.Immediate;
-				this.Queryable.ThisScheduler.Add (task);
+				this.queryable.ThisScheduler.Add (task);
 			}
 
-			EvolutionMailQueryable.log.Debug ("{0}: Finished indexing {1} ({2}/{3} {4:###.0}%)",
-							  this.folder_name, this.indexed_count, this.count,
+			string progress = "";
+			if (this.count > 0 && this.summary.header.count > 0) {
+				progress = String.Format (" ({0}/{1} {2:###.0}%)",
+							  this.count,
 							  this.summary.header.count,
 							  100.0 * this.count / this.summary.header.count);
+			}
+
+			Logger.Log.Debug ("{0}: Finished indexing {1} messages{2}", this.folder_name, this.indexed_count, progress);
 
 			this.SaveCache ();
 			this.CrawlFinished ();
@@ -782,32 +671,6 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			return false;
 		}
 
-		private MultiReader GetMessageData (string file)
-		{
-			try {
-				InitializeGMime ();
-			} catch (Exception e) {
-				Logger.Log.Warn ("Caught exception trying to initialize gmime:");
-				Logger.Log.Warn (e);
-				return null;
-			}
-
-			int fd = Syscall.open (file, OpenFlags.O_RDONLY);
-			GMime.StreamFs stream = new GMime.StreamFs (fd);
-			GMime.Parser parser = new GMime.Parser (stream);
-
-			MultiReader reader = null;
-
-			using (GMime.Message message = parser.ConstructMessage ())
-				reader = PartHandler.GetReader (message);
-
-			stream.Close ();
-			stream.Dispose ();
-			parser.Dispose ();
-
-			return reader;
-		}
-
 		// Kind of nasty, but we need the function.
 		[System.Runtime.InteropServices.DllImport("libglib-2.0.so.0")]
 		static extern int g_str_hash (string str);
@@ -824,8 +687,9 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			++this.count;
 
 			// Try to load the cached message data off disk
-			if (this.mapping[mi.uid] == null || (uint) mapping[mi.uid] != mi.flags) {
-				TextReader msgReader = null;
+			object flags = this.mapping[mi.uid];
+
+			if (flags == null || (uint) flags != mi.flags) {
 				string msg_file;
 
 				if (this.backend_type == ImapBackendType.Imap)
@@ -837,22 +701,17 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 					string cache_path = String.Format ("cache/{0:x}/{1}", hash, mi.uid);
 					msg_file = Path.Combine (summary_info.DirectoryName, cache_path);
 				}
-				
-				// FIXME - Filters really need to be rearchitected so that we can
-				// pass multiple attachments of different types into the indexable.
-				if (File.Exists (msg_file))
-					msgReader = GetMessageData (msg_file);
 
-				indexable = this.CamelMessageToIndexable (mi, msgReader);
+				indexable = this.CamelMessageToIndexable (mi, msg_file);
 
 				this.mapping[mi.uid] = mi.flags;
-				++this.indexed_count;
-			} 
 
-			this.deleted_list.Remove (mi.uid);
-			
-			// HACK: update your recipients
-			EvolutionMailQueryable.AddAsYourRecipient (indexable);
+				if (indexable != null) {
+					++this.indexed_count;
+					this.deleted_list.Remove (mi.uid);
+				}
+			} else if (flags != null)
+				this.deleted_list.Remove (mi.uid);
 
 			return indexable;
 		}
@@ -862,41 +721,79 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			return EvolutionMailQueryable.EmailUri (this.account_name, this.folder_name, message_info.uid);
 		}
 
-		private Indexable CamelMessageToIndexable (Camel.MessageInfo messageInfo, TextReader msgReader)
+		private Indexable CamelMessageToIndexable (Camel.MessageInfo messageInfo, string msg_file)
 		{
+			// Don't index messages flagged as junk
+			if (messageInfo.IsJunk)
+				return null;
+
+			// Many properties will be set by the filter when
+			// processing the cached data, if it's there.  So
+			// don't set a number of properties in that case.
+			bool have_content = File.Exists (msg_file);
+
 			Uri uri = CamelMessageUri (messageInfo);
 			Indexable indexable = new Indexable (uri);
 
 			indexable.Timestamp = messageInfo.Date;
-			indexable.Type = "MailMessage";
-
-			indexable.AddProperty (Property.New ("dc:title", messageInfo.subject));
-			indexable.AddProperty (Property.New ("fixme:subject",  messageInfo.subject));
+			indexable.MimeType = "message/rfc822";
+			indexable.HitType = "MailMessage";
 
 			indexable.AddProperty (Property.NewKeyword ("fixme:account",  this.imap_name));
                         indexable.AddProperty (Property.NewKeyword ("fixme:folder",   this.folder_name));
-			indexable.AddProperty (Property.NewKeyword ("fixme:subject",  messageInfo.subject));
-                        indexable.AddProperty (Property.NewKeyword ("fixme:to",       messageInfo.to));
-			indexable.AddProperty (Property.NewKeyword ("fixme:from",     messageInfo.from));
-                        indexable.AddProperty (Property.NewKeyword ("fixme:cc",       messageInfo.cc));
+			indexable.AddProperty (Property.NewKeyword ("fixme:client", "evolution"));
+			
+			if (!have_content) {
+				indexable.AddProperty (Property.New ("dc:title", messageInfo.subject));
+				indexable.AddProperty (Property.NewDate ("fixme:date", messageInfo.Date));
+			}
+
+			GMime.InternetAddressList addrs;
+			addrs = GMime.InternetAddressList.ParseString (messageInfo.to);
+			foreach (GMime.InternetAddress ia in addrs) {
+				if (!have_content) {
+					indexable.AddProperty (Property.NewKeyword ("fixme:to", ia.ToString (false)));
+					indexable.AddProperty (Property.NewKeyword ("fixme:to_address", ia.Addr));
+					indexable.AddProperty (Property.New ("fixme:to_name", ia.Name));
+				}
+
+				if (this.folder_name == "Sent")
+					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
+			}
+			addrs.Dispose ();
+
+			addrs = GMime.InternetAddressList.ParseString (messageInfo.cc);
+			foreach (GMime.InternetAddress ia in addrs) {
+				if (!have_content) {
+					indexable.AddProperty (Property.NewKeyword ("fixme:cc", ia.ToString (false)));
+					indexable.AddProperty (Property.NewKeyword ("fixme:cc_address", ia.Addr));
+					indexable.AddProperty (Property.New ("fixme:cc_name", ia.Name));
+				}
+
+				if (this.folder_name == "Sent")
+					indexable.AddProperty (Property.NewKeyword ("fixme:sentTo", ia.Addr));
+			}
+			addrs.Dispose ();
+
+			addrs = GMime.InternetAddressList.ParseString (messageInfo.from);
+			foreach (GMime.InternetAddress ia in addrs) {
+				if (!have_content) {
+					indexable.AddProperty (Property.NewKeyword ("fixme:from", ia.ToString (false)));
+					indexable.AddProperty (Property.NewKeyword ("fixme:from_address", ia.Addr));
+					indexable.AddProperty (Property.New ("fixme:from_name", ia.Name));
+				}
+
+				if (this.folder_name != "Sent")
+					indexable.AddProperty (Property.NewKeyword ("fixme:gotFrom", ia.Addr));
+			}
+			addrs.Dispose ();
+
                         indexable.AddProperty (Property.NewKeyword ("fixme:mlist",    messageInfo.mlist));
                         indexable.AddProperty (Property.NewKeyword ("fixme:flags",    messageInfo.flags));
 
-			if (messageInfo.received != DateTime.MinValue)
-				indexable.AddProperty (Property.NewDate ("fixme:received", messageInfo.received));
-
-			if (messageInfo.sent != DateTime.MinValue)
-				indexable.AddProperty (Property.NewDate ("fixme:sentdate", messageInfo.sent));
-
-			if (this.folder_name == "Sent") {
+			if (this.folder_name == "Sent")
 				indexable.AddProperty (Property.NewFlag ("fixme:isSent"));
 
-				MapAddressStringToProperties (indexable, "fixme:sentTo", messageInfo.to);
-				MapAddressStringToProperties (indexable, "fixme:sentTo", messageInfo.cc);
-			} else {
-				MapAddressStringToProperties (indexable, "fixme:gotFrom", messageInfo.from);
-			}
-
 			if (messageInfo.IsAnswered)
 				indexable.AddProperty (Property.NewFlag ("fixme:isAnswered"));
 
@@ -912,15 +809,15 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			if (messageInfo.IsSeen)
 				indexable.AddProperty (Property.NewFlag ("fixme:isSeen"));
 
-			if (messageInfo.HasAttachments)
+			if (messageInfo.HasAttachments && !have_content)
 				indexable.AddProperty (Property.NewFlag ("fixme:hasAttachments"));
 
 			if (messageInfo.IsAnsweredAll)
 				indexable.AddProperty (Property.NewFlag ("fixme:isAnsweredAll"));
 
-			if (msgReader != null)
-				indexable.SetTextReader (msgReader);
-			else
+			if (have_content)
+				indexable.ContentUri = UriFu.PathToFileUri (msg_file);
+			else 
 				indexable.NoContent = true;
 
 			return indexable;
@@ -928,10 +825,22 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 		public override void Checkpoint ()
 		{
-			EvolutionMailQueryable.log.Debug ("{0}: indexed {1} messages ({2}/{3} {4:###.0}%)",
-							  this.folder_name, this.indexed_count, this.count,
-							  this.summary.header.count,
-							  100.0 * this.count / this.summary.header.count);
+			if (this.summary != null) {
+
+				string progress = "";
+				if (this.count > 0 && this.summary.header.count > 0) {
+					progress = String.Format (" ({0}/{1} {2:###.0}%)",
+								  this.count,
+								  this.summary.header.count,
+								  100.0 * this.count / this.summary.header.count);
+
+					this.queryable.SetGeneratorProgress (this, (int) (100.0 * this.count / this.summary.header.count));
+
+				}
+				
+				Logger.Log.Debug ("{0}: indexed {1} messages{2}",
+						  this.folder_name, this.indexed_count, progress);
+			}
 
 			this.SaveCache ();
 		}
