@@ -37,16 +37,26 @@ namespace Beagle.Daemon {
 
 	public class FileAttributesStore_Sqlite : IFileAttributesStore {
 
-		const int VERSION = 1;
+		// Version history:
+		// 1: Original version
+		// 2: Replaced LastIndexedTime with LastAttrTime
+		const int VERSION = 2;
 
 		private SqliteConnection connection;
-		private byte[] path_flags;
+		private BitArray path_flags;
+		private int transaction_count = 0;
+
+		enum TransactionState {
+			None,
+			Requested,
+			Started
+		}
+		private TransactionState transaction_state;
 
 		public FileAttributesStore_Sqlite (string directory, string index_fingerprint)
 		{
 			bool create_new_db = false;
-
-			path_flags = new byte [8192];
+			path_flags = new BitArray (65536);
 
 			if (! File.Exists (GetDbPath (directory))) {
 				create_new_db = true;
@@ -101,7 +111,7 @@ namespace Beagle.Daemon {
 					    "  directory      STRING NOT NULL,        " +
 					    "  filename       STRING NOT NULL,        " +
 					    "  last_mtime     STRING NOT NULL,        " +
-					    "  last_indexed   STRING NOT NULL,        " +
+					    "  last_attrtime  STRING NOT NULL,        " +
 					    "  filter_name    STRING NOT NULL,        " +
 					    "  filter_version STRING NOT NULL         " +
 					    ")");
@@ -124,7 +134,7 @@ namespace Beagle.Daemon {
 					string dir = reader.GetString (0);
 					string file = reader.GetString (1);
 					string path = Path.Combine (dir, file);
-					SetPathFlag (path, true);
+					SetPathFlag (path);
 					++count;
 				}
 
@@ -133,7 +143,7 @@ namespace Beagle.Daemon {
 
 				DateTime dt2 = DateTime.Now;
 
-				Logger.Log.Info ("Loaded {0} records from {1} in {2:0.000}s", 
+				Logger.Log.Debug ("Loaded {0} records from {1} in {2:0.000}s", 
 						 count, GetDbPath (directory), (dt2 - dt1).TotalSeconds);
 			}
 		}
@@ -149,7 +159,8 @@ namespace Beagle.Daemon {
 		{
 			SqliteConnection c;			
 			c = new SqliteConnection ();
-			c.ConnectionString = "URI=file:" + GetDbPath (directory);
+			c.ConnectionString = "version=" + ExternalStringsHack.SqliteVersion
+				+ ",URI=file:" + GetDbPath (directory);
 			c.Open ();
 			return c;
 		}
@@ -182,7 +193,7 @@ namespace Beagle.Daemon {
 			command = new SqliteCommand ();
 			command.Connection = connection;
 			command.CommandText =
-				"SELECT unique_id, directory, filename, last_mtime, last_indexed, filter_name, filter_version " +
+				"SELECT unique_id, directory, filename, last_mtime, last_attrtime, filter_name, filter_version " +
 				"FROM file_attributes WHERE " + 
 				String.Format (where_format, where_args);
 			return command;
@@ -225,10 +236,13 @@ namespace Beagle.Daemon {
 			attr.UniqueId = GuidFu.FromShortString (reader.GetString (0));
 			attr.Path = System.IO.Path.Combine (reader.GetString (1), reader.GetString (2));
 			attr.LastWriteTime = StringFu.StringToDateTime (reader.GetString (3));
-			attr.LastIndexedTime = StringFu.StringToDateTime (reader.GetString (4));
+			attr.LastAttrTime = StringFu.StringToDateTime (reader.GetString (4));
 			attr.FilterName = reader.GetString (5);
 			attr.FilterVersion = int.Parse (reader.GetString (6));
 
+			if (attr.FilterName == "")
+				attr.FilterName = null;
+
 			return attr;
 		}
 
@@ -246,21 +260,13 @@ namespace Beagle.Daemon {
 		private bool GetPathFlag (string path)
 		{
 			int hash = GetPathHash (path);
-			int index = hash >> 3;
-			byte mask = (byte) (1 << (hash & 0x7));
-			return (path_flags [index] & mask) != 0;
+			return path_flags [hash];
 		}
 
-		private void SetPathFlag (string path, bool value)
+		private void SetPathFlag (string path)
 		{
 			int hash = GetPathHash (path);
-			int index = hash >> 3;
-			byte mask = (byte) (1 << (hash & 0x7));
-
-			if (value)
-				path_flags [index] |= mask;
-			else
-				path_flags [index] &= (byte) ~mask;
+			path_flags [hash] = true;
 		}
 
 		///////////////////////////////////////////////////////////////////
@@ -310,19 +316,29 @@ namespace Beagle.Daemon {
 
 		public bool Write (FileAttributes fa)
 		{
-			SetPathFlag (fa.Path, true);
+			SetPathFlag (fa.Path);
 
 			// We need to quote any 's that appear in the strings
 			// (in particular, in the path)
 			lock (connection) {
+				
+				// If a transaction has been requested, start it now.
+				MaybeStartTransaction ();
+
+				string filter_name;
+				filter_name = fa.FilterName;
+				if (filter_name == null)
+					filter_name = "";
+				filter_name = filter_name.Replace ("'", "''");
+
 				DoNonQuery ("INSERT OR REPLACE INTO file_attributes " +
-					    " (unique_id, directory, filename, last_mtime, last_indexed, filter_name, filter_version) " +
+					    " (unique_id, directory, filename, last_mtime, last_attrtime, filter_name, filter_version) " +
 					    " VALUES ('{0}', '{1}', '{2}', '{3}', '{4}', '{5}', '{6}')",
 					    GuidFu.ToShortString (fa.UniqueId),
 					    fa.Directory.Replace ("'", "''"), fa.Filename.Replace ("'", "''"),
 					    StringFu.DateTimeToString (fa.LastWriteTime),
-					    StringFu.DateTimeToString (fa.LastIndexedTime),
-					    fa.FilterName,
+					    StringFu.DateTimeToString (fa.LastAttrTime),
+					    filter_name,
 					    fa.FilterVersion);
 			}
 			return true;
@@ -330,7 +346,7 @@ namespace Beagle.Daemon {
 
 		public void Drop (string path)
 		{
-			// We don't want to SetPathFlag (path, false) here, since we have no way of knowing
+			// We don't want to "UnSetPathFlag" here, since we have no way of knowing
 			// if another path hashes to the same value as this one.
 
 			// We need to quote any 's that appear in the strings
@@ -338,9 +354,91 @@ namespace Beagle.Daemon {
 			string directory = Path.GetDirectoryName (path).Replace ("'", "''");
 			string filename = Path.GetFileName (path).Replace ("'", "''");
 			lock (connection) {
+
+				// If a transaction has been requested, start it now.
+				MaybeStartTransaction ();
+
 				DoNonQuery ("DELETE FROM file_attributes WHERE directory='{0}' AND filename='{1}'",
 					    directory, filename);
 			}
 		}
+
+		private void MaybeStartTransaction ()
+		{
+			if (transaction_state == TransactionState.Requested) {
+				DoNonQuery ("BEGIN");
+				transaction_state = TransactionState.Started;
+			}
+		}
+
+		public void BeginTransaction ()
+		{
+			if (transaction_state == TransactionState.None)
+				transaction_state = TransactionState.Requested;
+		}
+
+		public void CommitTransaction ()
+		{
+			if (transaction_state == TransactionState.Started) {
+				lock (connection)
+					DoNonQuery ("COMMIT");
+			}
+			transaction_state = TransactionState.None;
+		}
+
+		public void Flush ()
+		{
+			lock (connection) {
+				if (transaction_count > 0) {
+					Logger.Log.Debug ("Flushing requested -- committing sqlite transaction");
+					DoNonQuery ("COMMIT");
+					transaction_count = 0;
+				}
+			}
+		}
+
+		///////////////////////////////////////////////////////////////////
+
+		// Return all attributes in the attributes database, used for merging
+
+		private ICollection ReadAllAttributes () 
+		{
+			ArrayList attributes = new ArrayList ();
+			
+			SqliteCommand command;
+			SqliteDataReader reader;
+				
+			lock (connection) {
+				command = new SqliteCommand ();
+				command.Connection = connection;
+				command.CommandText =
+					"SELECT unique_id, directory, filename, last_mtime, last_attrtime, filter_name, filter_version " +
+					"FROM file_attributes";
+				
+				reader = ExecuteReaderOrWait (command);
+				
+				while (ReadOrWait (reader)) {
+					attributes.Add (GetFromReader (reader));
+				}
+				reader.Close ();
+				command.Dispose ();
+			}
+			
+			return attributes;
+		}
+
+		// FIXME: Might wanna do this a bit more intelligently
+
+		public void Merge (FileAttributesStore_Sqlite fa_sqlite_store_to_merge) 
+		{
+			ICollection attributes = fa_sqlite_store_to_merge.ReadAllAttributes ();
+				
+			BeginTransaction ();
+
+			foreach (FileAttributes attribute in attributes)
+				Write (attribute);
+
+			CommitTransaction ();
+		}
 	}
 }
