@@ -101,6 +101,7 @@ namespace Beagle.Daemon {
 
 		private string this_mime_type = null;
 		private string this_extension = null;
+		private ArrayList indexable_properties = null;
 
 		public string MimeType {
 			get { return this_mime_type; }
@@ -112,6 +113,13 @@ namespace Beagle.Daemon {
 			set { this_extension = value; }
 		}
 
+		// allow the filter to access the properties
+		// set by indexable
+		public ArrayList IndexableProperties {
+			get { return indexable_properties; }
+			set { indexable_properties = value; }
+		}
+		
 		//////////////////////////
 		
 		private bool crawl_mode = false;
@@ -208,19 +216,21 @@ namespace Beagle.Daemon {
 		public void AppendText (string str, string strHot)
 		{
 			if (!IsFrozen && str != null && str != "") {
-				
-				// FIXME: Do we need to handle '\n' in any 
-				// other way?
 				int i = 0;
-				string line;
-				string[] lines = str.Split ('\n');
-				for (i = 0; i < lines.Length; i++) {
-					line = lines[i].Trim();
-					if (line.Length > 0) {
-						ReallyAppendText (line, null);
-						AppendStructuralBreak ();
+				string[] lines;
+
+				// Avoid unnecessary allocation of a string
+				// FIXME: Handle \r, \r\n cases.
+				if (str.IndexOf ('\n') > -1) {
+					lines = str.Split ('\n'); 
+					foreach (string line in lines) {
+						if (line.Length > 0) {
+							ReallyAppendText (line, null);
+							AppendStructuralBreak ();
+						}
 					}
-				}
+				} else 
+					ReallyAppendText (str, null);
 			}
 			ReallyAppendText (null, strHot);
 		}
@@ -373,7 +383,7 @@ namespace Beagle.Daemon {
                         // Make sure the temporary file is only readable by the owner.
                         // FIXME: There is probably a race here.  Could some malicious program
                         // do something to the file between creation and the chmod?
-                        Mono.Unix.Syscall.chmod (tempFile, (Mono.Unix.FilePermissions) 256);
+                        Mono.Unix.Native.Syscall.chmod (tempFile, (Mono.Unix.Native.FilePermissions) 256);
 
                         BufferedStream buffered_stream = new BufferedStream (file_stream);
                         StreamWriter writer = new StreamWriter (buffered_stream);
@@ -407,7 +417,7 @@ namespace Beagle.Daemon {
                         // Make sure the temporary file is only readable by the owner.
                         // FIXME: There is probably a race here.  Could some malicious program
                         // do something to the file between creation and the chmod?
-                        Mono.Unix.Syscall.chmod (tempFile, (Mono.Unix.FilePermissions) 256);
+                        Mono.Unix.Native.Syscall.chmod (tempFile, (Mono.Unix.Native.FilePermissions) 256);
 
                         BufferedStream buffered_stream = new BufferedStream (file_stream);
 
