@@ -270,6 +270,9 @@ namespace Beagle.Daemon {
 				
 				// Set the filter's URI
 				candidate_filter.Uri = indexable.Uri;
+
+				// allow the filter access to the indexable's properties
+				candidate_filter.IndexableProperties = indexable.Properties;
 				
 				// Open the filter, copy the file's properties to the indexable,
 				// and hook up the TextReaders.
@@ -345,8 +348,10 @@ namespace Beagle.Daemon {
 
 					filter_versions_by_name [t.ToString ()] = filter.Version;
 
-					foreach (FilterFlavor flavor in filter.SupportedFlavors)
+					foreach (FilterFlavor flavor in filter.SupportedFlavors) {
 						filter_types_by_flavor [flavor] = t;
+						FilterFlavor.Flavors.Add (flavor);
+					}
 
 					++count;
 				}
