@@ -31,8 +31,6 @@ using System.IO;
 using SNS = System.Net.Sockets;
 using System.Threading;
 
-using Mono.Posix;
-
 using Gtk;
 
 using Beagle.Daemon;
@@ -42,16 +40,36 @@ namespace Beagle.IndexHelper {
 	
 	class IndexHelperTool {
 
+		static DateTime last_activity;
 		static Server server;
 
 		static void Main (string [] args)
+		{
+			try {
+				DoMain (args);
+			} catch (Exception ex) {
+				Logger.Log.Error ("Unhandled exception thrown.  Exiting immediately.");
+				Logger.Log.Error (ex);
+				Environment.Exit (1);
+			}
+		}
+
+		static void DoMain (string [] args)
 		{
 			bool run_by_hand = (Environment.GetEnvironmentVariable ("BEAGLE_RUN_HELPER_BY_HAND") != null);
 			bool log_in_fg = (Environment.GetEnvironmentVariable ("BEAGLE_LOG_IN_THE_FOREGROUND_PLEASE") != null);
+			
+			// FIXME: We always turn on full debugging output!  We are still
+			// debugging this code, after all...
+			//bool debug = (Environment.GetEnvironmentVariable ("BEAGLE_DEBUG_FLAG_IS_SET") != null);
 
-			Logger.DefaultLevel = LogLevel.Debug;
+			last_activity = DateTime.Now;
 
-			Logger.LogToFile (PathFinder.LogDir, "IndexHelper", run_by_hand || log_in_fg);
+			Log.Initialize (PathFinder.LogDir,
+					"IndexHelper",
+					//debug ? LogLevel.Debug : LogLevel.Warn,
+					LogLevel.Debug,
+					run_by_hand || log_in_fg);
 
 			Application.InitCheck ("IndexHelper", ref args);
 
@@ -72,29 +90,54 @@ namespace Beagle.IndexHelper {
 			}
 
 			if (server_has_been_started) {
+				// Set the IO priority to idle so we don't slow down the system
+				if (Environment.GetEnvironmentVariable ("BEAGLE_EXERCISE_THE_DOG") == null)
+					IoPriority.SetIdle ();
 				
-				// Start the monitor thread, which keeps an eye on memory usage.
-				ExceptionHandlingThread.Start (new ThreadStart (MemoryMonitorWorker));
+				// Start the monitor thread, which keeps an eye on memory usage and idle time.
+				ExceptionHandlingThread.Start (new ThreadStart (MemoryAndIdleMonitorWorker));
 
 				// Start a thread that watches the daemon and begins a shutdown
 				// if it terminates.
 				ExceptionHandlingThread.Start (new ThreadStart (DaemonMonitorWorker));
 
 				Application.Run ();
-			}
 
-			Environment.Exit (0);
+				// If we palced our sockets in a temp directory, try to clean it up
+				// Note: this may fail because the daemon is still running
+				if (PathFinder.GetRemoteStorageDir (false) != PathFinder.StorageDir) {
+					try {
+						Directory.Delete (PathFinder.GetRemoteStorageDir (false));
+					} catch (IOException) { }
+				}
+			}
 		}
 
-		static void MemoryMonitorWorker ()
+		static public void ReportActivity ()
+		{
+			last_activity = DateTime.Now;
+		}
+
+		static void MemoryAndIdleMonitorWorker ()
 		{
 			int vmrss_original = SystemInformation.VmRss;
+
+			const double max_idle_time = 30; // minutes
+
 			const double threshold = 5.0;
 			const int max_request_count = 0;
 			int last_vmrss = 0;
 
 			while (! Shutdown.ShutdownRequested) {
 
+				double idle_time;
+				idle_time = (DateTime.Now - last_activity).TotalMinutes;
+				if (idle_time > max_idle_time && RemoteIndexerExecutor.Count > 0) {
+					Logger.Log.Debug ("No activity for {0:0.0} minutes, shutting down", idle_time);
+					Shutdown.BeginShutdown ();
+					return;
+				}
+
 				// Check resident memory usage
 				int vmrss = SystemInformation.VmRss;
 				double size = vmrss / (double) vmrss_original;
@@ -107,27 +150,37 @@ namespace Beagle.IndexHelper {
 					if (RemoteIndexerExecutor.Count > 0) {
 						Logger.Log.Debug ("Process too big, shutting down!");
 						Shutdown.BeginShutdown ();
+						return;
 					} else {
 						// Paranoia: don't shut down if we haven't done anything yet
 						Logger.Log.Debug ("Deferring shutdown until we've actually done something.");
-						Thread.Sleep (250);
+						Thread.Sleep (1000);
 					}
 				} else {
-					Thread.Sleep (1000);
+					Thread.Sleep (3000);
 				}
 			}
 		}
 		
 		static void DaemonMonitorWorker ()
 		{
+			string storage_dir = PathFinder.GetRemoteStorageDir (false);
+
+			if (storage_dir == null) {
+				Logger.Log.Debug ("The daemon doesn't appear to have started");
+				Logger.Log.Debug ("Shutting down helper.");
+				Shutdown.BeginShutdown ();
+				return;
+			}
+
 			// FIXME: We shouldn't need to know the  name of the daemon's socket.
 			string socket_name;
-			socket_name = Path.Combine (PathFinder.StorageDir, "socket");
+			socket_name = Path.Combine (storage_dir, "socket");
 
 			try {
 				SNS.Socket socket;
 				socket = new SNS.Socket (SNS.AddressFamily.Unix, SNS.SocketType.Stream, 0);
-				socket.Connect (new UnixEndPoint (socket_name));
+				socket.Connect (new Mono.Unix.UnixEndPoint (socket_name));
 				
 				ArrayList socket_list = new ArrayList ();
 				
@@ -149,25 +202,16 @@ namespace Beagle.IndexHelper {
 
 		/////////////////////////////////////////////////////////////////////////////
 
-		// The integer values of the Mono.Posix.Signal enumeration don't actually
-		// match the Linux signal numbers of Linux.  Oops!
-		// This is fixed in Mono.Unix, but for the moment we want to maintain
-		// compatibility with mono 1.0.x.
-		const int ACTUAL_LINUX_SIGINT  = 2;
-		const int ACTUAL_LINUX_SIGQUIT = 3;
-		const int ACTUAL_LINUX_SIGTERM = 15;
-
 		static void SetupSignalHandlers ()
 		{
 			// Force OurSignalHandler to be JITed
 			OurSignalHandler (-1);
 
 			// Set up our signal handler
-			Mono.Posix.Syscall.sighandler_t sig_handler;
-			sig_handler = new Mono.Posix.Syscall.sighandler_t (OurSignalHandler);
-                        Mono.Posix.Syscall.signal (ACTUAL_LINUX_SIGINT, sig_handler);
-                        Mono.Posix.Syscall.signal (ACTUAL_LINUX_SIGQUIT, sig_handler);
-                        Mono.Posix.Syscall.signal (ACTUAL_LINUX_SIGTERM, sig_handler);
+			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGINT, OurSignalHandler);
+			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGTERM, OurSignalHandler);
+			if (Environment.GetEnvironmentVariable("BEAGLE_THERE_BE_NO_QUITTIN") == null)
+				Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGQUIT, OurSignalHandler);
 		}
 
 		// Our handler triggers an orderly shutdown when it receives a signal.
@@ -214,9 +258,9 @@ namespace Beagle.IndexHelper {
 
 		static void OnShutdown ()
 		{
-			server.Stop ();
+			if (server != null)
+				server.Stop ();
 		}
-
 	}
 
 }
