@@ -33,6 +33,8 @@ using Beagle.Util;
 namespace Beagle.Daemon {
 
 	public abstract class LuceneQueryable : IQueryable {
+		
+		static public bool OptimizeRightAway = false;
 
 		public delegate IIndexer IndexerCreator (string name, int minor_version);
 
@@ -165,10 +167,8 @@ namespace Beagle.Daemon {
 
 		virtual public bool AcceptQuery (Query query)
 		{
-			// Don't accept queries on empty indexes.
-			// If nothing else, it causes us to start
-			// up a new thread.
-			return GetItemCount () > 0;
+			// Accept all queries by default.
+			return true;
 		}
 
 		/////////////////////////////////////////
@@ -185,13 +185,6 @@ namespace Beagle.Daemon {
 
 		/////////////////////////////////////////
 
-		virtual protected Hit PostProcessHit (Hit hit)
-		{
-			return hit;
-		}
-
-		/////////////////////////////////////////
-
 		// DEPRECATED: This does nothing, since everything is now
 		// time-based.
 		virtual protected double RelevancyMultiplier (Hit hit)
@@ -282,10 +275,17 @@ namespace Beagle.Daemon {
 			
 			ICollection added_uris = null;
 
+			// Index listeners never return any initial matches.
+			if (change_data == null && query.IsIndexListener)
+				return;
+
 			if (change_data != null) {
 				
-				if (change_data.RemovedUris != null) 
+				if (change_data.RemovedUris != null) {
+					foreach (Uri uri in change_data.RemovedUris)
+						Log.Debug ("Signalling removal of {0}", uri);
 					query_result.Subtract (change_data.RemovedUris);
+				}
 
 				// If nothing was added, we can safely return now: this change
 				// cannot have any further effect on an outstanding live query.
@@ -294,6 +294,24 @@ namespace Beagle.Daemon {
 					return;
 
 				added_uris = change_data.AddedUris;
+
+				// If this is an index listener, we don't need to do a query:
+				// we just build up synthethic hits and add them unconditionally.
+				if (query.IsIndexListener) {
+					ArrayList synthetic_hits = new ArrayList ();
+					foreach (Uri uri in added_uris) {
+						if (our_uri_filter != null && ! our_uri_filter (uri))
+							continue;
+						Hit hit = new Hit ();
+						hit.Uri = uri;
+						if (our_hit_filter != null && ! our_hit_filter (hit))
+							continue;
+						synthetic_hits.Add (hit);
+					}
+					if (synthetic_hits.Count > 0)
+						query_result.Add (synthetic_hits);
+					return;
+				}
 			}
 			
 			Driver.DoQuery (query, 
@@ -330,14 +348,56 @@ namespace Beagle.Daemon {
 
 		/////////////////////////////////////////
 
-		public virtual int GetItemCount ()
+		private int progress_percent = -1;
+		private QueryableState state = QueryableState.Idle;
+		private DateTime last_state_change = DateTime.MinValue;
+
+		public QueryableStatus GetQueryableStatus ()
 		{
-			// If we're in read-only mode, query the driver and 
-			// not the indexer for the item count.
+			QueryableStatus status = new QueryableStatus ();
+
+			status.State = state;
+			status.ProgressPercent = progress_percent;
+
+			// If we're in read-only mode, query the driver
+			// and not the indexer for the item count.
 			if (indexer == null)
-				return driver.GetItemCount ();
+				status.ItemCount = driver.GetItemCount ();
 			else
-				return indexer.GetItemCount ();
+				status.ItemCount = indexer.GetItemCount ();
+
+			// Frequent state changes are common, and there isn't
+			// a real state machine with continuity when it comes
+			// to the indexing process.  A delayed indexing task,
+			// for example, might not actually run for several
+			// seconds after it is scheduled.  In this case, the
+			// backend might be in an "Idle" state, but the
+			// indexing process clearly isn't done.  To work
+			// around this, we also track the last time the state
+			// changed.  If it's less than some threshold, then
+			// we consider ourselves to still be in the process of
+			// indexing.
+			if (state != QueryableState.NotApplicable
+			    && (state != QueryableState.Idle
+				|| (DateTime.Now - last_state_change).TotalSeconds <= 30))
+				status.IsIndexing = true;
+
+			return status;
+		}
+
+		public QueryableState State {
+			get { return this.state; }
+			set { 
+				//Logger.Log.Debug ("State {0}: {1} -> {2}", this, this.state, value);
+
+				this.state = value;
+				this.last_state_change = DateTime.Now;
+			}
+		}
+
+		public int ProgressPercent {
+			get { return this.progress_percent; }
+			set { this.progress_percent = value; }
 		}
 
 		/////////////////////////////////////////
@@ -435,6 +495,9 @@ namespace Beagle.Daemon {
 
 			override protected void DoTaskReal ()
 			{
+				QueryableState old_state = queryable.State;
+				queryable.State = QueryableState.Indexing;
+
 				if (queryable.PreAddIndexableHook (indexable)) {
 					queryable.AddIndexable (indexable);
 
@@ -443,6 +506,8 @@ namespace Beagle.Daemon {
 					else
 						queryable.ConditionalFlush ();
 				}
+
+				queryable.State = old_state;
 			}
 
 			override protected void DoCleanup ()
@@ -486,6 +551,9 @@ namespace Beagle.Daemon {
 				// get re-scheduled after it is run.
 				Reschedule = true;
 
+				QueryableState old_state = queryable.State;
+				queryable.State = QueryableState.Indexing;
+
 				do {
 					if (! generator.HasNextIndexable ()) {
 						// ...except if there is no more work to do, of course.
@@ -501,7 +569,7 @@ namespace Beagle.Daemon {
 					// to return this time through, but it does not mean that
 					// its processing queue is empty.
 					if (generated == null)
-						break;
+						continue;
 
 					if (queryable.PreAddIndexableHook (generated))
 						queryable.AddIndexable (generated);
@@ -510,6 +578,12 @@ namespace Beagle.Daemon {
 				} while (! queryable.ConditionalFlush ());
 
 				generator.PostFlushHook ();
+
+				queryable.State = old_state;
+			}
+
+			override protected void DoCleanup ()
+			{
 			}
 		}
 
@@ -673,7 +747,7 @@ namespace Beagle.Daemon {
 			if (our_optimize_task == null)
 				our_optimize_task = NewOptimizeTask ();
 
-			if (Environment.GetEnvironmentVariable ("BEAGLE_UNDER_BLUDGEON") != null)
+			if (OptimizeRightAway || Environment.GetEnvironmentVariable ("BEAGLE_UNDER_BLUDGEON") != null)
 				optimize_delay = 1/120.0; // half a second
 
 			// Changing the trigger time of an already-scheduled process
@@ -689,8 +763,7 @@ namespace Beagle.Daemon {
 		// Other hooks
 
 		// If this returns true, a task will automatically be created to
-		// add the child.  Note that the PreAddHook will also be called,
-		// as usual.
+		// add the child.
 		virtual protected bool PreChildAddHook (Indexable child)
 		{
 			return true;
@@ -707,6 +780,8 @@ namespace Beagle.Daemon {
 
 		protected void AddIndexable (Indexable indexable)
 		{
+			indexable.Source = QueryDriver.GetQueryable (this).Name;
+
 			lock (request_lock)
 				pending_request.Add (indexable);
 
@@ -726,22 +801,42 @@ namespace Beagle.Daemon {
 		// Returns true if we actually did flush, false otherwise.
 		protected bool ConditionalFlush ()
 		{
-			lock (request_lock) {
-				if (pending_request.Count > 37) { // a total arbitrary magic number
-					Flush ();
-					return true;
+			QueryableState old_state = State;
+			State = QueryableState.Flushing;
+
+			try {
+				lock (request_lock) {
+					if (pending_request.Count > 37) { // a total arbitrary magic number
+						Flush ();
+						return true;
+					}
 				}
+				return false;
+			} finally {
+				State = old_state;
 			}
-			return false;
 		}
 
 		protected void Flush ()
+		{
+			QueryableState old_state = State;
+			State = QueryableState.Flushing;
+
+			try {
+				DoFlush ();
+			} finally {
+				State = old_state;
+			}
+		}
+
+		private void DoFlush ()
 		{
 			IndexerRequest flushed_request;
 
 			lock (request_lock) {
 				if (pending_request.IsEmpty)
 					return;
+
 				flushed_request = pending_request;
 				pending_request = new IndexerRequest ();
 
@@ -765,6 +860,9 @@ namespace Beagle.Daemon {
 			if (receipts.Length == 0)
 				return;
 
+			// Update the cached count of items in the driver
+			driver.SetItemCount (indexer.GetItemCount ());
+
 			// Something happened, so schedule an optimize just in case.
 			ScheduleOptimize ();
 			
