@@ -55,7 +55,6 @@ struct _BeagleQueryClass {
 
 	void (*hits_added)      (BeagleQuery *query, BeagleHitsAddedResponse *response);
 	void (*hits_subtracted) (BeagleQuery *query, BeagleHitsSubtractedResponse *response);
-	void (*cancelled)       (BeagleQuery *query);
 	void (*finished)        (BeagleQuery *query, BeagleFinishedResponse *response);
 };
 
@@ -78,5 +77,8 @@ void	     beagle_query_set_max_hits (BeagleQuery *query,
 					int max_hits);
 int          beagle_query_get_max_hits (BeagleQuery *query);
 
+GSList      *beagle_query_get_exact_text   (BeagleQuery *query);
+GSList      *beagle_query_get_stemmed_text (BeagleQuery *query);
+
 #endif /* __BEAGLE_QUERY_H */
 
